/*----------------------------------------------------------------------
  File    : PatternSet.java
  Contents: pattern set(s) for training of a multilayer perceptrons
  Author  : Christian Borgelt
  History : 2023.04.29 file created from file MLP.c
----------------------------------------------------------------------*/
package jmlp;

/*--------------------------------------------------------------------*/
/** Class for very simple training patterns.
 *  @author Christian Borgelt
 *  @since  2023.04.30 */
/*--------------------------------------------------------------------*/
public class PatternSet {

  /*------------------------------------------------------------------*/
  /*  constants                                                       */
  /*------------------------------------------------------------------*/
  private static final long serialVersionUID = 0x00010000L;

  /** problem type: exclusive or */
  public static final int XOR    = 0;
  /** problem type: biimplication */
  public static final int BIIMP  = 1;
  /** problem type: function 1 */
  public static final int FUNC1  = 2;
  /** problem type: function 2 */
  public static final int FUNC2  = 3;

  /** training patterns for exclusive or */
  static final double[][] xor = {
    { 0, 0, 0 }, { 0, 1, 1 }, { 1, 0, 1 }, { 1, 1, 0 } };

  /** training patterns for biimplication */
  static final double[][] biimp = {
    { 0, 0, 1 }, { 0, 1, 0 }, { 1, 0, 0 }, { 1, 1, 1 } };

  /** training patterns for function 1 */
  static final double[][] func1 = {
    { -1.0, 0, 0.1  }, { -0.8, 0, 0.1  }, { -0.6, 0, 0.3  },
    { -0.4, 0, 0.5  }, { -0.2, 0, 0.7  }, {  0.0, 0, 0.9  },
    {  0.2, 0, 0.9  }, {  0.4, 0, 0.9  }, {  0.5, 0, 0.7  },
    {  0.6, 0, 0.5  }, {  0.7, 0, 0.3  }, {  0.8, 0, 0.1  },
    {  1.0, 0, 0.1  } };

  /** training patterns for function 2 */
  static final double[][] func2 = {
    { -1.0, 0, 0.1  }, { -0.8, 0, 0.1  }, { -0.6, 0, 0.18 },
    { -0.4, 0, 0.26 }, { -0.2, 0, 0.34 }, {  0.0, 0, 0.42 },
    {  0.2, 0, 0.5  }, {  0.4, 0, 0.7  }, {  0.6, 0, 0.9  },
    {  0.8, 0, 0.9  }, {  1.0, 0, 0.9  } };

  /*------------------------------------------------------------------*/
  /*  instance variables                                              */
  /*------------------------------------------------------------------*/
  /** the problem type (e.g. XOR or FUNC1) */
  protected int        type;
  /** the number of inputs (1 or 2) */
  protected int        inpcnt;
  /** the array of training patterns */
  protected double[][] pats;

  /*------------------------------------------------------------------*/
  /** Create a pattern set.
   *  @param  type  the problem type identifier
   *  @since  2023.04.30 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public PatternSet (int type)
  {                             /* --- create a pattern set */
    this.type = ((type >= XOR) && (type <= FUNC2)) ? type : XOR;
    switch (this.type) {        /* evaluate the problem type */
      case XOR:   this.inpcnt = 2; this.pats = xor;   break;
      case BIIMP: this.inpcnt = 2; this.pats = biimp; break;
      case FUNC1: this.inpcnt = 1; this.pats = func1; break;
      case FUNC2: this.inpcnt = 1; this.pats = func2; break;
    }                           /* set inputs and patterns */
  }  /* PatternSet() */

  /*------------------------------------------------------------------*/
  /** Get the problem type of the pattern set.
   *  @return the problem type of the pattern set
   *  @since  2023.04.30 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int type ()
  { return this.type; }

  /*------------------------------------------------------------------*/
  /** Get the number of inputs in each pattern.
   *  @return the number of inputs in each pattern
   *  @since  2023.04.30 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int inpcnt ()
  { return this.inpcnt; }

  /*------------------------------------------------------------------*/
  /** Get the number of patterns (size of the pattern set).
   *  @return the number of patterns (size of the pattern set)
   *  @since  2023.04.30 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int size ()
  { return this.pats.length; }

  /*------------------------------------------------------------------*/
  /** Get a training patterns.
   *  @param  i the index of the training pattern
   *  @return the training pattern as an array of numbers
   *  @since  2023.04.30 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double[] pattern (int i)
  { return this.pats[i]; }

}  /* class PatternSet */   
